<?php
declare (strict_types = 1);
namespace app\user\controller;

use app\BaseController;
use f2fpay\service\AlipayCertifyService;
use app\common\library\Pub;
use think\facade\Db;
use app\common\library\Auth;
use think\facade\Config;

/**
 * 实名认证
 */
class Certification extends BaseController
{



    public function initialize() {
        $web = Config::get('website');
        if (!isset($web['status']))exitContent(view(config('app.maintain_tmpl'), ['webqq' => $web['webqq'],'webname' => $web['webname'],'cdn' => $web['cdn']]));
        $this->alipay_config = array_merge(Config::get('alipay'),Config::get('website'));
        $this->alipay_config['cert_return_url'] = 'alipays://platformapi/startapp?appId=20000067&url='.urlencode(request()->domain() . url('user/certification/certok'));
        if (empty($this->alipay_config['alipay_public_key'])) {
            exitContent(view(config('app.dispatch_success_tmpl'), [
                'code' => 0,
                'msg' => '支付宝公钥没有配置',
                'data' => '',
                'url' => 'javascript:history.back(-1);',
                'wait' => '3',
            ]));
        }
        if (empty($this->alipay_config['cert_channel'] == '1')) {

            exitContent(view(config('app.dispatch_success_tmpl'), [
                'code' => 0,
                'msg' => '实名认证未开启',
                'data' => '',
                'url' => 'javascript:history.back(-1);',
                'wait' => '3',
            ]));
        }
        if (empty($this->alipay_config['private_key'])) {

            exitContent(view(config('app.dispatch_success_tmpl'), [
                'code' => 0,
                'msg' => '商户私钥没有配置',
                'data' => '',
                'url' => 'javascript:history.back(-1);',
                'wait' => '3',
            ]));
        }
        if (empty($this->alipay_config['app_id'])) {

            exitContent(view(config('app.dispatch_success_tmpl'), [
                'code' => 0,
                'msg' => '应用ID没有配置',
                'data' => '',
                'url' => 'javascript:history.back(-1);',
                'wait' => '3',
            ]));
        }
    }

    public function certok() {
        if (!session('username')&&!session('userid')) {
            $this->error('请先登录');
        }
        return '<!DOCTYPE html>
<html>
<head>
    <meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
    <meta charset="utf-8" />
    <meta name="viewport" content="initial-scale=1, maximum-scale=1, user-scalable=no, width=device-width">
    <link href="//cdn.bootcss.com/ionic/1.3.1/css/ionic.min.css" rel="stylesheet" />
</head>
<body>
<div class="bar bar-header bar-light" align-title="center">
	<h1 class="title">实名认证结果页面</h1>
</div>
<div class="has-header" style="padding: 5px;position: absolute;width: 100%;">
<div class="text-center" style="color: #a09ee5;">
<i class="icon ion-checkmark-circled" style="font-size: 80px;"></i><br>
<span>实名认证成功，请关闭此页面</span>
</div>
</div>
<script>
document.querySelector("body").addEventListener("touchmove", function (event) {
	event.preventDefault();
});
</script>
</body>
</html>';
    }

    public function certificate() {
        if (!session('username')&&!session('userid')) {
            $this->error('请先登录');
        }
        if (!empty(input('get.certify_id'))) {
            $certify_id = input('get.certify_id','','htmlspecialchars');
            if (empty(session(session('userid').'_certify_id')) || session(session('userid').'_certify_id') !== $certify_id) {
                return  redirect(url('user/certification/certificate'));
            }
            $this->assign('certify_id',$certify_id);
            $this->assign('qrcode_url',request()->domain() . url('user/certification/alipaycert').'?id='.$certify_id);
        }
        $csrf_token = md5(mt_rand(0,999).time());
        session('csrf_token',$csrf_token);
        $this->assign('csrf_token',$csrf_token);
        $user = Db::table('user')->where(array('username' => session('username')))->find();
        $this->assign('user',$user);
        $this->assign('web', $this->alipay_config);
        return view('user/'.$this->alipay_config['user_template'].'/html/certificate');
    }
    public function alipaycert() {
        if (empty(input('get.id'))) {
            $this->error('非法请求');
        }
        $certify_id = input('get.id');
        $certify = new AlipayCertifyService($this->alipay_config);
        $html = $certify->certify($certify_id);
        return '<!DOCTYPE html>
<html>
<head>
    <meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
    <meta charset="utf-8" />
    <meta name="viewport" content="initial-scale=1, maximum-scale=1, user-scalable=no, width=device-width">
	<title>正在跳转</title>
	<style type="text/css">
        body {margin:0;padding:0;}
        p {position:absolute;
            left:50%;top:50%;
            width:330px;height:30px;
            margin:-35px 0 0 -160px;
            padding:20px;font:bold 14px/30px "宋体", Arial;
            background:#f9fafc url(../assets/img/loading.gif) no-repeat 20px 20px;
            text-indent:40px;border:1px solid #c5d0dc;}
        #waiting {font-family:Arial;}
    </style>
</head>
<body>
<p>正在跳转，请稍候...</p>
        '.$html.'
</body>
</html>';
    }
    public function ajax_certificate() {
        if (!session('username')&&!session('userid')) {
            $this->error('请先登录');
        }
        $user = Db::table('user')->where(array('username' => session('username')))->find();
        $certname = input('post.certname','','htmlspecialchars');
        $certno = input('post.certno','','htmlspecialchars');
        if (!input('post.csrf_token') || input('post.csrf_token') != session('csrf_token'))$this->error('CSRF TOKEN ERROR');
        if ($user['cert'] == 1)$this->error('你已完成实名认证');
        if ($this->alipay_config['cert_money'] > 0 && $user['usermoney'] < $this->alipay_config['cert_money'])$this->error('账户余额不足'.$this->alipay_config['cert_money'].'元，无法完成认证');
        if (empty($certname) || empty($certno))$this->error('请确保各项不能为空');
        if (strlen($certname) < 3)$this->error('姓名填写错误');
        if (!Pub::is_idcard($certno))$this->error('身份证号不正确');
        $certify = new AlipayCertifyService($this->alipay_config);
        $outer_order_no = date("YmdHis").rand(000,999).$uid;
        $certifyResult = $certify->initialize($outer_order_no, $certname, $certno, 'SMART_FACE');
        if (isset($certifyResult['certify_id'])) {
            session(session('userid').'_certify_id',$certifyResult['certify_id']);

            if (!Db::table('user')->where(array('username' => session('username')))->update(['certno' => (new Auth)->encrypt($certno),'certname' => (new Auth)->encrypt($certname)])) {
                $this->error('保存信息失败');
            }
            return ['code' => '1','msg' => 'succ','certify_id' => $certifyResult['certify_id'],'url' => request()->domain() . url('user/certification/alipaycert').'?id='.$certifyResult['certify_id']];
        } else {
            $this->error('支付宝接口返回异常['.$certifyResult['sub_code'].']'.$certifyResult['sub_msg'].'');
        }
    }
    public function ajax_query() {
        if (!session('username')&&!session('userid')) {
            $this->error('请先登录');
        }
        $certify_id = input('post.certify_id');
        if (empty($certify_id)) {
            $this->error('非法请求');
        }
        $user = Db::table('user')->where(array('username' => session('username')))->find();
        if (!input('post.csrf_token') || input('post.csrf_token') != session('csrf_token'))$this->error('CSRF TOKEN ERROR');
        if (!empty(session(session('userid').'_certify_id')) && session(session('userid').'_certify_id') == $certify_id) {
            $certify = new AlipayCertifyService($this->alipay_config);
            $certifyResult = $certify->query($certify_id);
            if (isset($certifyResult['passed'])) {
                if ($certifyResult['passed'] == 'T') {
                    Db::table('user')->where(array('username' => session('username')))->update(['cert' => 1,'certtime' => date("Y-m-d H:i:s")]);
                    if ($this->alipay_config['cert_money'] > 0) {
                        Db::table('user')->where(array('username' => session('username')))->update(['usermoney' => $user['usermoney'] - $conf['cert_money']]);
                    }
                    return ['code' => 1,'msg' => 'succ','passed' => true];
                } else {
                    return ['code' => 1,'msg' => 'succ','passed' => false];
                }
            } else {
                $this->error('支付宝接口返回异常['.$certifyResult['sub_code'].']'.$certifyResult['sub_msg'].'');
            }
        } else {
            $this->error('Access Denied');
        }
    }
}