<?php
declare (strict_types = 1);
namespace app\user\controller;

use app\BaseController;
use f2fpay\model\builder\AlipayTradePrecreateContentBuilder;
use f2fpay\model\builder\AlipayTradeQueryContentBuilder;
use f2fpay\model\builder\AopClient;
use f2fpay\service\AlipayTradeService;
use app\common\library\Pub;
use think\facade\Db;
use think\facade\Config;

/**
 * 当面付
 */
class F2fpay extends BaseController
{



    public function initialize() {
        $web = Config::get('website');
        if (!isset($web['status']))exitContent(view(config('app.maintain_tmpl'), ['webqq' => $web['webqq'],'webname' => $web['webname'],'cdn' => $web['cdn']]));
        $this->alipay_config = array_merge(Config::get('alipay'),Config::get('website'));
        $this->alipay_config['notify_url'] = request()->domain() . url('user/f2fpay/notify');
        if (empty($this->alipay_config['alipay_public_key'])) {
            exitContent(view(config('app.dispatch_success_tmpl'), [
                'code' => 0,
                'msg' => '支付宝公钥没有配置',
                'data' => '',
                'url' => 'javascript:history.back(-1);',
                'wait' => '3',
            ]));
        }
        if (empty($this->alipay_config['alipay'] == 'f2f')) {

            exitContent(view(config('app.dispatch_success_tmpl'), [
                'code' => 0,
                'msg' => '当面付未开启',
                'data' => '',
                'url' => 'javascript:history.back(-1);',
                'wait' => '3',
            ]));
        }
        if (empty($this->alipay_config['private_key'])) {

            exitContent(view(config('app.dispatch_success_tmpl'), [
                'code' => 0,
                'msg' => '商户私钥没有配置',
                'data' => '',
                'url' => 'javascript:history.back(-1);',
                'wait' => '3',
            ]));
        }
        if (empty($this->alipay_config['app_id'])) {

            exitContent(view(config('app.dispatch_success_tmpl'), [
                'code' => 0,
                'msg' => '应用ID没有配置',
                'data' => '',
                'url' => 'javascript:history.back(-1);',
                'wait' => '3',
            ]));
        }
    }

    public function query() {
        if (!session('username')&&!session('userid')) {
            $this->error('请先登录');
        }
        if (input('post.no')) {
            $out_trade_no = input('post.no');
            $queryContentBuilder = new AlipayTradeQueryContentBuilder();
            $queryContentBuilder->setOutTradeNo($out_trade_no);

            $queryResponse = new AlipayTradeService($this->alipay_config);
            $queryResult = $queryResponse->queryTradeResult($queryContentBuilder);
            $res['status'] = $queryResult->getTradeStatus();
            $res['buyer'] = isset($queryResult->getResponse()->buyer_logon_id) ? $queryResult->getResponse()->buyer_logon_id : '';
            $res['amount'] = $queryResult->getResponse()->buyer_pay_amount;
                
            if ($res['status'] == 'SUCCESS') {
                return $this->paySuccess($out_trade_no);
               
            }

            exit(json_encode($res));
        } else {
            $this->error('非法请求');
        }
    }

    // 等待支付页面
    public function pay() {
        if (!session('username')&&!session('userid')) {
            $this->error('请先登录');
        }
        if (empty(input('post.body')) || empty(input('post.no')) || empty(input('post.money')) || empty(input('post.qr'))) {
            $this->error('非法请求');
        } else {
            $this->assign('body', input('post.body'));
            $this->assign('out_trade_no', input('post.no'));
            $this->assign('money', input('post.money'));

            $this->assign('qr', input('post.qr'));

            $this->assign('web', $this->alipay_config);
             return view('user/'.$this->alipay_config['user_template'].'/html/f2fpay');
        }
    }

    /**
     * 支付入口
     * @Author   Jason
     * @DateTime 2020-12-12
     * @param    [type]     $out_trade_no 订单号
     * @param    [type]     $money        支付金额
     * @param    [type]     $body         标题（备注）小于20字
     * @return   [type]                   [description]
     */
    public function toPay($out_trade_no, $money) {
        if (!session('username')&&!session('userid')) {
            $this->error('请先登录');
        }
        $body = $this->alipay_config['webname'].'在线支付';

        $totalAmount = round(trim($money), 2);
        //支付宝金额只支持两位小数
        if (!is_numeric($totalAmount) || $totalAmount == 0) {
            $this->error('订单金额不合法');
        }


        // 创建请求builder，设置请求参数
        $qrPayRequestBuilder = new AlipayTradePrecreateContentBuilder();
        $qrPayRequestBuilder->setOutTradeNo($out_trade_no);
        $qrPayRequestBuilder->setTotalAmount($totalAmount);
        $qrPayRequestBuilder->setSubject($body);
        #print_r(get_class_methods($qrPayRequestBuilder));
        // 调用qrPay方法获取当面付应答

        // var_dump( $alipay_config);exit;
        $qrPay = new AlipayTradeService($this->alipay_config);
        $qrPayResult = $qrPay->qrPay($qrPayRequestBuilder);

        if ($qrPayResult->getTradeStatus() === "SUCCESS") {
            $qr = $qrPayResult->getResponse()->qr_code;
            //SUCCESS 是官方SDK给的结果。如果想看详细的介绍，去找SDK
        } else {
            $this->error('生成订单失败,错误原因：' . $qrPayResult->getTradeStatus());
        }

        $form['body'] = $body;
        $form['no'] = $out_trade_no;
        $form['money'] = $totalAmount;
        $form['qr'] = $qr;
        // var_dump($form);exit;
        $sHtml = "<form id='alipaysubmit' name='alipaysubmit' action='" . url('user/F2fpay/pay') . "' method='POST'>";
        // var_dump($sHtml);exit;
        foreach ($form as $key => $value) {
            if ($value != null) {
                //$value = $this->characet($value, $this->postCharset);
                $value = str_replace("'", "&apos;", $value);
                //$value = str_replace("\"","&quot;",$value);
                $sHtml .= "<input type='hidden' name='" . $key . "' value='" . $value . "'/>";
            }
        }

        //submit按钮控件请不要含有name属性
        $sHtml = $sHtml . "<input type='submit' value='ok' style='display:none;''></form>";

        $sHtml = $sHtml . "<script>document.forms['alipaysubmit'].submit();</script>";
        exit($sHtml);
    }

    // 异步回调
    public function notify() {
        if ($this->request->isPost()) {
            $aop = new AopClient;
            $aop->alipayrsaPublicKey = $this->alipay_config['alipay_public_key'];
            $flag = $aop->rsaCheckV1($_POST, null, "RSA2");
            if ($flag) {
                echo 'success';
                //接口必须返回success 不然阿里会一直发送校验验证。
                //异步SIGN验证成功， 可以进行下一步动作。例如验证订单金额 然后完成订单。之类的。。
                //需要验证的就是 订单号 与 订单金额是否一致，验证成功 就可以对数据库中的订单进行操作了。
                //TRADE_SUCCESS 对于当面付来说，已经到账了。详情可以看这里 https://www.cnblogs.com/tdalcn/p/5956690.html
                if ($_POST['trade_status'] === "TRADE_SUCCESS") {
                    //订单处理模板
                    $no = $_POST['out_trade_no'];
                    $mount = $_POST['total_amount'];
                    $trade_no = $_POST['trade_no'];
                    $notify_time = $_POST['notify_time'];
                    $buyer_logon_id = $_POST['buyer_logon_id'];
                    return $this->paySuccess($no, $trade_no);
                }
            } else {
                echo 'error';
            }
        } else {
            exit();
        }
    }



    /**
     * 获取订单信息
     * @Author   Jason
     * @DateTime 2020-12-12
     * @param    [type]     $out_trade_no [description]
     * @return   [type]                   [description]
     */
    private function get_payinfo($out_trade_no) {
        if ($payinfo = Db::table('pay')->where('out_trade_no', $out_trade_no)->find()) {
            return $payinfo;
        } else {
            return false;
        }
    }

    /**
     * 支付成功
     * @Author   Jason
     * @DateTime 2020-12-12
     * @return   [type]        [description]
     */
    private function paySuccess($out_trade_no, $trade_no = '') {


        // 查找订单
        $payInfo = $this->get_payinfo($out_trade_no);
        // 没有找到该订单
        if (!$payInfo) {
            return 'null';
        }

        // 支付成功
        if ($payInfo['state'] == 1) {
            return 'success';
        }
        $payInfo['money'] = $payInfo['money'] + $payInfo['money'] * ($this->alipay_config['payfree'] / 100);
        // 写入数据库
        $up = Db::table('pay')->where('out_trade_no', $out_trade_no)->update(['state' => 1]);
        $user = Db::table('user')->where(array('username' => $payInfo['username']))->find();
        if (!Db::table('user')->where(array('username' => $payInfo['username']))->update(['usermoney' => $user['usermoney'] + $payInfo['money']])) {
            return 'error';
        }
        if (!$up) {

            return 'error';
        }
        Pub::LingduMsg('充值成功提醒！','充值金额：'.$payInfo['money'].'<br/>现有余额：'.($user['usermoney'] + $payInfo['money']).'<br/>充值用户：'.$payInfo['username'].'<br/>欢迎您下次充值！',$payInfo['username']);
        Pub::LingduMsgs('用户充值提醒！','用户充值金额：'.$payInfo['money'].'<br/>用户目前余额：'.($user['usermoney'] + $payInfo['money']).'<br/>充值用户：'.$payInfo['username']);
        return 'success';
    }
}